package de.ozgcloud.nachrichten.postfach.osiv2.config;

import java.util.Objects;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.env.Environment;
import org.springframework.security.oauth2.client.AuthorizedClientServiceReactiveOAuth2AuthorizedClientManager;
import org.springframework.security.oauth2.client.InMemoryReactiveOAuth2AuthorizedClientService;
import org.springframework.security.oauth2.client.ReactiveOAuth2AuthorizedClientProvider;
import org.springframework.security.oauth2.client.ReactiveOAuth2AuthorizedClientProviderBuilder;
import org.springframework.security.oauth2.client.endpoint.WebClientReactiveClientCredentialsTokenResponseClient;
import org.springframework.security.oauth2.client.registration.ReactiveClientRegistrationRepository;
import org.springframework.security.oauth2.client.web.reactive.function.client.ServerOAuth2AuthorizedClientExchangeFilterFunction;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.reactive.function.client.WebClient;

import lombok.RequiredArgsConstructor;

@Configuration
@RequiredArgsConstructor
public class WebClientConfiguration {

	private final Environment environment;

	@Bean("osi2PostfachWebClient")
	public WebClient osi2PostfachWebClient(
			ServerOAuth2AuthorizedClientExchangeFilterFunction serverOAuth2AuthorizedClientExchangeFilterFunction) {
		var url = Objects.requireNonNull(
				environment.getProperty("ozgcloud.osiv2-postfach.api.url"),
				"ozgcloud.osiv2-postfach.api.url is not set");
		return WebClient.builder()
				.baseUrl(url)
				.filter(serverOAuth2AuthorizedClientExchangeFilterFunction)
				.build();
	}

	@Bean
	@Primary
	ServerOAuth2AuthorizedClientExchangeFilterFunction serverOAuth2AuthorizedClientExchangeFilterFunction(
			ReactiveClientRegistrationRepository clientRegistrations) {

		var oauth = new ServerOAuth2AuthorizedClientExchangeFilterFunction(authorizedClientManager(clientRegistrations));
		oauth.setDefaultClientRegistrationId("osi2");
		return oauth;
	}

	AuthorizedClientServiceReactiveOAuth2AuthorizedClientManager authorizedClientManager(
			ReactiveClientRegistrationRepository clientRegistrations) {
		var clientService = new InMemoryReactiveOAuth2AuthorizedClientService(
				clientRegistrations);
		var authorizedClientManager = new AuthorizedClientServiceReactiveOAuth2AuthorizedClientManager(
				clientRegistrations, clientService);

		authorizedClientManager.setAuthorizedClientProvider(authorizedClientProvider());

		return authorizedClientManager;
	}

	ReactiveOAuth2AuthorizedClientProvider authorizedClientProvider() {
		return ReactiveOAuth2AuthorizedClientProviderBuilder.builder()
				.clientCredentials(builder ->
						builder.accessTokenResponseClient(getClientCredentialsTokenResponseClient())
				)
				.build();
	}

	WebClientReactiveClientCredentialsTokenResponseClient getClientCredentialsTokenResponseClient() {
		var client = new WebClientReactiveClientCredentialsTokenResponseClient();
		var resource = Objects.requireNonNull(
				environment.getProperty("ozgcloud.osiv2-postfach.api.resource"),
				"ozgcloud.osiv2-postfach.api.resource is not set"
		);

		client.addParametersConverter(source -> {
			MultiValueMap<String, String> parameters = new LinkedMultiValueMap<>();
			// Pass a resource indicator parameter https://datatracker.ietf.org/doc/html/rfc8707
			parameters.add("resource", resource);
			return parameters;
		});
		return client;
	}

}
