package de.ozgcloud.nachrichten.postfach.osiv2.extension;

import org.junit.jupiter.api.extension.AfterAllCallback;
import org.junit.jupiter.api.extension.AfterEachCallback;
import org.junit.jupiter.api.extension.BeforeAllCallback;
import org.junit.jupiter.api.extension.ExtensionContext;
import org.mockserver.client.MockServerClient;
import org.testcontainers.containers.MockServerContainer;
import org.testcontainers.containers.output.OutputFrame;
import org.testcontainers.utility.DockerImageName;

import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Getter
@RequiredArgsConstructor
public class OsiMockServerExtension implements BeforeAllCallback, AfterAllCallback, AfterEachCallback {

	private MockServerClient mockClient;
	private final MockServerContainer mockServerContainer = new MockServerContainer(DockerImageName.parse("mockserver/mockserver")
			.withTag("mockserver-5.15.0"))
			.withLogConsumer(outputFrame -> {
				var line = outputFrame.getUtf8String().stripTrailing();
				if (outputFrame.getType() == OutputFrame.OutputType.STDERR) {
					LOG.error(line);
				} else {
					LOG.info(line);
				}
			});

	@Override
	public void beforeAll(ExtensionContext context) {
		setupMockServer();
	}

	@Override
	public void afterEach(ExtensionContext context) {
		mockClient.reset();
	}

	@Override
	public void afterAll(ExtensionContext context) {
		mockServerContainer.stop();
		mockClient.stop();
	}

	private void setupMockServer() {
		mockServerContainer.start();
		mockClient = new MockServerClient(
				mockServerContainer.getHost(),
				mockServerContainer.getServerPort()
		);
	}

	public String getAccessTokenUrl() {
		return getMockServerUrl() + "/access-token";
	}

	public String getPostfachFacadeUrl() {
		return getMockServerUrl();
	}

	private String getMockServerUrl() {
		return "http://" + mockClient.remoteAddress().getHostName() + ":" + mockClient.remoteAddress().getPort();
	}

}
